import 'dart:async';

import 'package:flutter/material.dart';

import 'package:provider/provider.dart';
import 'package:puzzlegame/provider/data.dart';
import 'package:puzzlegame/screens/game.dart';
import 'package:puzzlegame/utils/game_utils.dart';
import 'package:puzzlegame/utils/get_images_list.dart';

class Home extends StatelessWidget {
  static const String id = 'home';

  final Color themePrimaryColor = Colors.blue;
  final int _selectImageColumnsCount = 3;

  Future<void> getImagesList(Data myProvider) async {
    GetImagesList getImagesList;
    getImagesList = GetImagesList();
    await getImagesList.init();
    if (getImagesList.availableImages.isNotEmpty) {
      myProvider.updateAvailableImages(getImagesList.availableImages);
      shuffleAvailableImages(myProvider);
    }
  }

  Future<void> selectImage(Data myProvider, String imageCode, double tileImageSize) async {
    myProvider.updateIsShufflingBoard(true);
    myProvider.updateSelectedImage(imageCode);
    GameUtils.startGame(myProvider, tileImageSize);
  }

  void shuffleAvailableImages(Data myProvider) {
    List images = myProvider.availableImages;
    images.shuffle();
    myProvider.updateAvailableImages(images);
  }

  Container _buildImageSelectorItem(Data myProvider, String image, double tileImageSize) {
    return Container(
      child: TextButton(
        style: TextButton.styleFrom(
          padding: EdgeInsets.all(2),
        ),
        child: Container(
          decoration: BoxDecoration(
            borderRadius: BorderRadius.circular(4),
            border: Border.all(
              color: Colors.blue.shade200,
              width: 4,
            ),
          ),
          child: Image(
            image: AssetImage(GameUtils.getImageAssetName(image)),
            fit: BoxFit.fill,
          ),
        ),
        onPressed: () {
          selectImage(myProvider, image, tileImageSize);
        },
      ),
    );
  }

  Container _buildImageSelector(Data myProvider, double tileImageSize) {
    if (myProvider.availableImages.length == 0) {
      getImagesList(myProvider);
    }
    List images = myProvider.availableImages;

    return Container(
      padding: EdgeInsets.all(2),
      child: ListView(
        children: [
          Table(
            defaultColumnWidth: IntrinsicColumnWidth(),
            children: [
              for (var imgIndex = 0;
                  imgIndex < images.length;
                  imgIndex += _selectImageColumnsCount)
                TableRow(
                  children: [
                    for (var colIndex = 0; colIndex < _selectImageColumnsCount; colIndex++)
                      Column(
                        children: [
                          if (imgIndex + colIndex < images.length)
                            _buildImageSelectorItem(
                                myProvider, images[imgIndex + colIndex], tileImageSize)
                        ],
                      ),
                  ],
                ),
            ],
          ),
        ],
      ),
    );
  }

  TextButton _buildTilesetSizeSelectorItem(Data myProvider, int value) {
    String assetName =
        'assets/icons/difficulty_' + value.toString() + 'x' + value.toString() + '.png';

    Color borderColor = themePrimaryColor;

    if (myProvider.tilesCount == value) {
      borderColor = Colors.white;
    }

    return TextButton(
      child: Container(
        decoration: BoxDecoration(
          borderRadius: BorderRadius.circular(4),
          border: Border.all(
            color: borderColor,
            width: 4,
          ),
        ),
        child: Image(
          image: AssetImage(assetName),
          fit: BoxFit.fill,
        ),
      ),
      onPressed: () {
        myProvider.updateTilesCount(value);
      },
    );
  }

  @override
  Widget build(BuildContext context) {
    Data _myProvider = Provider.of<Data>(context);

    var content;

    double _tileImageSize = (MediaQuery.of(context).size.width - 70) / _myProvider.tilesCount;

    if (_myProvider.isShufflingBoard) {
      content = Game.buildShufflingIndicatorWidget();
      precacheImage(new AssetImage('assets/icons/game_win.png'), context);
    } else {
      if (_myProvider.selectedImage == '') {
        content = _buildImageSelector(_myProvider, _tileImageSize);
      } else {
        content = Game.buildGameWidget(_myProvider);
      }
    }

    List<Widget> menuActions = [
      _buildTilesetSizeSelectorItem(_myProvider, 3),
      _buildTilesetSizeSelectorItem(_myProvider, 4),
      _buildTilesetSizeSelectorItem(_myProvider, 5),
      TextButton(
        child: Container(
          decoration: BoxDecoration(
            borderRadius: BorderRadius.circular(4),
            border: Border.all(
              color: themePrimaryColor,
              width: 4,
            ),
          ),
          child: Image(
            image: AssetImage('assets/icons/button_shuffle.png'),
            fit: BoxFit.fill,
          ),
        ),
        onPressed: () => shuffleAvailableImages(_myProvider),
      ),
    ];

    List<Widget> gameActions = [
      TextButton(
        child: Container(
          decoration: BoxDecoration(
            borderRadius: BorderRadius.circular(4),
            border: Border.all(
              color: Colors.blue,
              width: 4,
            ),
          ),
          child: Image(
            image: AssetImage('assets/icons/button_back.png'),
            fit: BoxFit.fill,
          ),
        ),
        onPressed: () => GameUtils.resetGame(_myProvider),
      ),
    ];

    return Scaffold(
      appBar: AppBar(
        backgroundColor: themePrimaryColor,
        actions: _myProvider.selectedImage == '' ? menuActions : gameActions,
      ),
      body: SafeArea(
        child: Center(child: content),
      ),
    );
  }
}
